/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: File Searching helper
//-----------------------------------------------------------------------------

#include "FileSearch.h"
#include "../String/SecureTcscat.h"

BOOL CFileSearch::IsDirectory(WIN32_FIND_DATA* pWin32FindData)
{
    return ((pWin32FindData->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0);
}

//-----------------------------------------------------------------------------
// Name: Search
// Object: search for files and directories. file name can contain * and ?
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
BOOL CFileSearch::Search(TCHAR* PathWithWildChar,BOOL SearchInSubDirectories,pfFileFoundCallBack FileFoundCallBack,PVOID UserParam)
{
    return CFileSearch::Search(PathWithWildChar,SearchInSubDirectories,NULL,FileFoundCallBack,UserParam);
}

BOOL CFileSearch::Search(TCHAR* PathWithWildChar,BOOL SearchInSubDirectories,HANDLE hCancelEvent,pfFileFoundCallBack FileFoundCallBack,PVOID UserParam)
{
    BOOL SearchCanceled;
    return CFileSearch::Search(PathWithWildChar,SearchInSubDirectories,hCancelEvent,FileFoundCallBack,UserParam,&SearchCanceled);
}

BOOL CFileSearch::Search(TCHAR* PathWithWildChar,BOOL SearchInSubDirectories,HANDLE hCancelEvent,pfFileFoundCallBack FileFoundCallBack,PVOID UserParam,BOOL* pSearchCanceled)
{
    TCHAR* pszFileFilter;
    // split search in directory + file filter
    pszFileFilter=_tcsrchr(PathWithWildChar,'\\');
    if (!pszFileFilter)
        return TRUE;
    // point after '\'
    pszFileFilter++;

    // get directory
    SIZE_T Size=pszFileFilter-PathWithWildChar;
    TCHAR szPath[MAX_PATH];
    Size = __min(Size,MAX_PATH-1);
    _tcsncpy(szPath,PathWithWildChar,Size);
    szPath[Size]=0;

    return CFileSearch::SearchMultipleNames(szPath,&pszFileFilter,1,SearchInSubDirectories,hCancelEvent,FileFoundCallBack,UserParam,pSearchCanceled);
}

BOOL CFileSearch::SearchMultipleNames(TCHAR* Directory, TCHAR** FileNameWithWildCharArray,SIZE_T FileNameWithWildCharArraySize,BOOL SearchInSubDirectories,HANDLE hCancelEvent,pfFileFoundCallBack FileFoundCallBack,PVOID UserParam,BOOL* pSearchCanceled)
{
    BOOL bRetValue;
    HANDLE hFileHandle=INVALID_HANDLE_VALUE;
    WIN32_FIND_DATA FindFileData={0};
    *pSearchCanceled=FALSE;
    bRetValue = FALSE;

    if (::IsBadCodePtr((FARPROC)FileFoundCallBack))
        return FALSE;

    // if cancel event is signaled
    if (::WaitForSingleObject(hCancelEvent,0)==WAIT_OBJECT_0)
        return TRUE;

    if (::IsBadReadPtr(Directory,sizeof(TCHAR)))
        return FALSE;

    if (::IsBadReadPtr(FileNameWithWildCharArray,sizeof(TCHAR*)*FileNameWithWildCharArraySize))
        return FALSE;

    SIZE_T Cnt;
    TCHAR* FullPath=NULL;
    TCHAR* SearchSubDirectoriesPattern=NULL;
    TCHAR* SubDirectory=NULL;
    SIZE_T FullPathSize;
    SIZE_T SearchSubDirectoriesPatternSize;
    SIZE_T SubDirectorySize;
    SIZE_T EndOfDirectory;


    FullPathSize = __max( (_tcslen(Directory)+1)*2, 1024 );
    FullPath = new TCHAR[FullPathSize];
    _tcscpy(FullPath,Directory);
    EndOfDirectory = _tcslen(FullPath);
    // assume that directory ends with '\\'
    if (Directory[EndOfDirectory-1]!='\\')
    {
        _tcscat(FullPath,_T("\\"));
        EndOfDirectory++;
    }

    //////////////////////////////////////////////
    // 1) find files in current directory
    //////////////////////////////////////////////

    for (Cnt=0;Cnt<FileNameWithWildCharArraySize;Cnt++)
    {
        if (::IsBadReadPtr(FileNameWithWildCharArray[Cnt],sizeof(TCHAR)))
        {
#ifdef _DEBUG
            if (::IsDebuggerPresent())
                ::DebugBreak();
#endif
            continue;
        }

        // remove last filename if any
        FullPath[EndOfDirectory]=0;

        // forge new path
        CSecureTcscat::Secure_tcscat(&FullPath,FileNameWithWildCharArray[Cnt],&FullPathSize);

        // find first file
        hFileHandle=::FindFirstFile(FullPath,&FindFileData);
        if (hFileHandle!=INVALID_HANDLE_VALUE)// if no matching file found
        {
            do 
            {
                // call callback
                if (!FileFoundCallBack(Directory,&FindFileData,UserParam))
                {
                    *pSearchCanceled=TRUE;
                    bRetValue = TRUE;
                    goto CleanUp;
                }

                // check cancel event
                if (hCancelEvent)
                {
                    // if event is signaled
                    if (::WaitForSingleObject(hCancelEvent,0)==WAIT_OBJECT_0)
                    {
                        *pSearchCanceled=TRUE;
                        bRetValue = TRUE;
                        goto CleanUp;
                    }
                }

                // find next file
            } while(::FindNextFile(hFileHandle, &FindFileData));

            ::FindClose(hFileHandle);
            // prepare for next goto cleanup
            hFileHandle = INVALID_HANDLE_VALUE;
        }
    }

    //////////////////////////////////////////////
    // 2) find sub directories
    //////////////////////////////////////////////

    // check if we should search in subdirectories
    if (!SearchInSubDirectories)
    {
        bRetValue = TRUE;
        goto CleanUp;
    }

    // remove last filename if any
    FullPath[EndOfDirectory]=0;

    // SearchSubDirectoriesPattern="path\*"
    SearchSubDirectoriesPatternSize = _tcslen(FullPath)+2;// 1 for \0 + 1 for '*'
    SearchSubDirectoriesPattern = new TCHAR[SearchSubDirectoriesPatternSize];
    _tcscpy(SearchSubDirectoriesPattern,FullPath);
    CSecureTcscat::Secure_tcscat(&SearchSubDirectoriesPattern,_T("*"),&SearchSubDirectoriesPatternSize);

    SubDirectorySize = _tcslen(FullPath)+1+260+1;//1 for \0, 260 for FindFileData.cFileName, 1 for '\\'
    SubDirectory= new TCHAR[SubDirectorySize];

    // find first file
    hFileHandle=::FindFirstFile(SearchSubDirectoriesPattern,&FindFileData);
    if (hFileHandle==INVALID_HANDLE_VALUE)// if no file found
    {
        bRetValue = TRUE;
        goto CleanUp;
    }

    do 
    {
        if ((FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0)
        {
            if (     (_tcscmp(FindFileData.cFileName,_T("."))==0)
                || (_tcscmp(FindFileData.cFileName,_T(".."))==0)
                )
                continue;

            // we have assume that SubDirectorySize is greater than FullPath
            _tcscpy(SubDirectory,FullPath);
            CSecureTcscat::Secure_tcscat(&SubDirectory, FindFileData.cFileName,&SubDirectorySize);
            CSecureTcscat::Secure_tcscat(&SubDirectory, _T("\\"),&SubDirectorySize);
            CFileSearch::SearchMultipleNames(SubDirectory,FileNameWithWildCharArray,FileNameWithWildCharArraySize,SearchInSubDirectories,hCancelEvent,FileFoundCallBack,UserParam,pSearchCanceled);
            if (*pSearchCanceled)
            {
                bRetValue = TRUE;
                goto CleanUp;
            }
        }

        // check cancel event
        if (hCancelEvent)
        {
            // if event is signaled
            if (::WaitForSingleObject(hCancelEvent,0)==WAIT_OBJECT_0)
            {
                *pSearchCanceled=TRUE;
                bRetValue = TRUE;
                goto CleanUp;
            }
        }

        // find next file
    } while(::FindNextFile(hFileHandle, &FindFileData));

    bRetValue = TRUE;
CleanUp:
    if (hFileHandle!=INVALID_HANDLE_VALUE)
        ::FindClose(hFileHandle);
    if (FullPath)
        delete[] FullPath;
    if (SearchSubDirectoriesPattern)
        delete[] SearchSubDirectoriesPattern;
    if (SubDirectory)
        delete[] SubDirectory;
    return bRetValue;
}